package com.greenorange.promotion.controller.project;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.greenorange.promotion.annotation.RequiresPermission;
import com.greenorange.promotion.annotation.SysLog;
import com.greenorange.promotion.common.BaseResponse;
import com.greenorange.promotion.common.ErrorCode;
import com.greenorange.promotion.common.ResultUtils;
import com.greenorange.promotion.constant.UserConstant;
import com.greenorange.promotion.exception.ThrowUtils;
import com.greenorange.promotion.model.dto.CommonRequest;
import com.greenorange.promotion.model.dto.project.ProjectAddRequest;
import com.greenorange.promotion.model.dto.project.ProjectQueryRequest;
import com.greenorange.promotion.model.dto.project.ProjectUpdateRequest;
import com.greenorange.promotion.model.dto.promoCodeApply.PromoCodeApplyRequest;
import com.greenorange.promotion.model.entity.*;
import com.greenorange.promotion.model.vo.project.ProjectCardVO;
import com.greenorange.promotion.model.vo.project.ProjectDetailVO;
import com.greenorange.promotion.model.vo.project.ProjectPageVO;
import com.greenorange.promotion.model.vo.project.ProjectVO;
import com.greenorange.promotion.model.vo.projectNotification.ProjectNotificationVO;
import com.greenorange.promotion.model.vo.userProject.UserProjectVO;
import com.greenorange.promotion.service.common.CommonService;
import com.greenorange.promotion.service.project.*;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * 项目 控制器
 */
@RestController
@RequestMapping("project")
@Slf4j
@Tag(name = "项目管理")
public class ProjectController {

    @Resource
    private ProjectService projectService;

    @Resource
    private CommonService commonService;

    @Resource
    private ProjectDetailService projectDetailService;

    @Resource
    private ProjectNotificationService projectNotificationService;

    @Resource
    private PromoCodeService promoCodeService;

    @Resource
    private ProjectCommissionService projectCommissionService;

    @Resource
    private SubUserProjectCommissionService subUserProjectCommissionService;

    @Resource
    private UserProjectService userProjectService;



    /**
     * 小程序用户查看项目列表
     * @return 项目列表
     */
    @PostMapping("query/card")
    @Operation(summary = "小程序用户查看项目列表", description = "参数：无，权限：管理员，方法名：queryProjectCardList")
    @RequiresPermission(mustRole = UserConstant.DEFAULT_ROLE)
    @SysLog(title = "项目管理", content = "小程序用户查看项目列表")
    public BaseResponse<List<ProjectCardVO>> queryProjectCardList() {
        List<Project> projectList = projectService.list();
        List<ProjectCardVO> projectCardVOS = commonService.convertList(projectList, ProjectCardVO.class);
        return ResultUtils.success(projectCardVOS);
    }

    /**
     * 小程序用户根据id查询项目详情
     * @param commonRequest 项目id
     * @return 项目详情
     */
    @PostMapping("query/id")
    @Operation(summary = "小程序用户根据id查询项目详情", description = "参数：无，权限：管理员，方法名：queryProjectDetailById")
    @RequiresPermission(mustRole = UserConstant.DEFAULT_ROLE)
    @SysLog(title = "项目管理", content = "小程序用户根据id查询项目详情")
    public BaseResponse<ProjectDetailVO> queryProjectDetailById(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        Project project = projectService.getById(id);
        ProjectDetailVO projectDetailVO = commonService.copyProperties(project, ProjectDetailVO.class);
        // 获取项目通知列表
        LambdaQueryWrapper<ProjectNotification> notificationLambdaQueryWrapper = new LambdaQueryWrapper<>();
        notificationLambdaQueryWrapper.eq(ProjectNotification::getProjectId, id);
        List<ProjectNotification> projectNotificationList = projectNotificationService.list(notificationLambdaQueryWrapper);
        List<ProjectNotificationVO> projectNotificationVOS = commonService.convertList(projectNotificationList, ProjectNotificationVO.class);
        // 获取项目明细列表
        LambdaQueryWrapper<ProjectDetail> projectDetailLambdaQueryWrapper = new LambdaQueryWrapper<>();
        projectDetailLambdaQueryWrapper.eq(ProjectDetail::getProjectId, id);
        List<ProjectDetail> projectDetailList = projectDetailService.list(projectDetailLambdaQueryWrapper);
        List<ProjectDetailVO> projectDetailVOS = commonService.convertList(projectDetailList, ProjectDetailVO.class);
        projectDetailVO.setProjectNotificationVOList(projectNotificationVOS);
        projectDetailVO.setProjectDetailVOList(projectDetailVOS);
        return ResultUtils.success(projectDetailVO);
    }


    /**
     * 小程序用户查看正在推广的项目
     * @return 推广的项目列表
     */
    @PostMapping("get/running")
    @Operation(summary = "小程序用户查看项目列表", description = "参数：无，权限：管理员，方法名：queryUserProjectList")
    @RequiresPermission(mustRole = UserConstant.DEFAULT_ROLE)
    @SysLog(title = "项目管理", content = "小程序用户查看项目列表")
    public BaseResponse<List<UserProjectVO>> queryUserProjectList(HttpServletRequest request) {
        Long userId = (Long) request.getAttribute("userId");
        LambdaQueryWrapper<UserProject> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserProject::getUserId, userId);
        List<UserProject> userProjectList = userProjectService.list(lambdaQueryWrapper);
        List<UserProjectVO> userProjectVOS = commonService.convertList(userProjectList, UserProjectVO.class);
        return ResultUtils.success(userProjectVOS);
    }



    /**
     * web端管理员添加项目
     * @param projectAddRequest 项目添加请求体
     * @return 是否添加成功
     */
    @PostMapping("add")
    @Operation(summary = "web端管理员添加项目", description = "参数：项目添加请求体，权限：管理员，方法名：addProject")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员添加项目")
    public BaseResponse<Boolean> addProject(@Valid @RequestBody ProjectAddRequest projectAddRequest) {
        Project project = commonService.copyProperties(projectAddRequest, Project.class);
        projectService.save(project);
        return ResultUtils.success(true);
    }

    /**
     * web端管理员根据id修改项目信息
     * @param projectUpdateRequest 项目更新请求体
     * @return 是否更新成功
     */
    @PostMapping("update")
    @Operation(summary = "web端管理员更新项目", description = "参数：项目更新请求体，权限：管理员，方法名：updateProject")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员根据id修改项目信息")
    public BaseResponse<Boolean> updateProject(@Valid @RequestBody ProjectUpdateRequest projectUpdateRequest) {
        Project project = commonService.copyProperties(projectUpdateRequest, Project.class);
        projectService.updateById(project);
        // 批量修改用户项目记录
        LambdaQueryWrapper<UserProject> userProjectLambdaQueryWrapper = new LambdaQueryWrapper<>();
        userProjectLambdaQueryWrapper.eq(UserProject::getProjectId, project.getId());
        List<UserProject> userProjectList = userProjectService.list(userProjectLambdaQueryWrapper);
        for (UserProject userProject : userProjectList) {
            userProject.setProjectName(project.getProjectName());
            userProject.setProjectImage(project.getProjectImage());
            userProject.setProjectSettlementCycle(project.getProjectSettlementCycle());
        }
        userProjectService.updateBatchById(userProjectList);
        return ResultUtils.success(true);
    }

    /**
     * web端管理员根据id删除项目
     * @param commonRequest 项目删除请求体
     * @return 是否删除成功
     */
    @PostMapping("delete")
    @Operation(summary = "web端管理员根据id删除项目", description = "参数：项目删除请求体，权限：管理员，方法名：delProject")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员根据id删除项目")
    public BaseResponse<Boolean> delProject(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        projectService.removeById(id);
        LambdaQueryWrapper<ProjectDetail> projectDetailLambdaQueryWrapper = new LambdaQueryWrapper<>();
        projectDetailLambdaQueryWrapper.eq(ProjectDetail::getProjectId, id);
        projectDetailService.remove(projectDetailLambdaQueryWrapper);
        LambdaQueryWrapper<ProjectNotification> projectNotificationLambdaQueryWrapper = new LambdaQueryWrapper<>();
        projectNotificationLambdaQueryWrapper.eq(ProjectNotification::getProjectId, id);
        projectNotificationService.remove(projectNotificationLambdaQueryWrapper);
        LambdaQueryWrapper<PromoCode> promoCodeLambdaQueryWrapper = new LambdaQueryWrapper<>();
        promoCodeLambdaQueryWrapper.eq(PromoCode::getProjectId, id);
        promoCodeService.remove(promoCodeLambdaQueryWrapper);
        LambdaQueryWrapper<ProjectCommission> projectCommissionLambdaQueryWrapper = new LambdaQueryWrapper<>();
        projectCommissionLambdaQueryWrapper.eq(ProjectCommission::getProjectId, id);
        projectCommissionService.remove(projectCommissionLambdaQueryWrapper);
        LambdaQueryWrapper<SubUserProjectCommission> subUserProjectCommissionLambdaQueryWrapper = new LambdaQueryWrapper<>();
        subUserProjectCommissionLambdaQueryWrapper.eq(SubUserProjectCommission::getProjectId, id);
        subUserProjectCommissionService.remove(subUserProjectCommissionLambdaQueryWrapper);
        LambdaQueryWrapper<UserProject> userProjectLambdaQueryWrapper = new LambdaQueryWrapper<>();
        userProjectLambdaQueryWrapper.eq(UserProject::getProjectId, id);
        userProjectService.remove(userProjectLambdaQueryWrapper);
        return ResultUtils.success(true);
    }

//    /**
//     * web端管理员批量删除项目
//     * @param commonBatchRequest 项目批量删除请求体
//     * @return 是否删除成功
//     */
//    @PostMapping("delBatch")
//    @Operation(summary = "web端管理员批量删除项目", description = "参数：项目批量删除请求体，权限：管理员，方法名：delBatchProject")
//    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
//    @SysLog(title = "项目管理", content = "web端管理员批量删除项目")
//    public BaseResponse<Boolean> delBatchProject(@Valid @RequestBody CommonBatchRequest commonBatchRequest) {
//        List<Long> ids = commonBatchRequest.getIds();
//        projectService.removeByIds(ids);
//        return ResultUtils.success(true);
//    }

    /**
     * web端管理员根据id查询项目
     * @param commonRequest 项目查询请求体
     * @return 项目信息
     */
    @PostMapping("queryById")
    @Operation(summary = "web端管理员根据id查询项目", description = "参数：项目查询请求体，权限：管理员，方法名：queryProjectById")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "web端管理员根据id查询项目")
    public BaseResponse<ProjectVO> queryProjectById(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        Project project = projectService.getById(id);
        ThrowUtils.throwIf(project == null, ErrorCode.OPERATION_ERROR, "当前项目不存在");
        ProjectVO projectVO = commonService.copyProperties(project, ProjectVO.class);
        return ResultUtils.success(projectVO);
    }

    /**
     * Web端管理员分页查询项目
     * @param projectQueryRequest 项目查询请求体
     * @return 项目列表
     */
    @PostMapping("page")
    @Operation(summary = "Web端管理员分页查询项目", description = "参数：项目查询请求体，权限：管理员，方法名：listProjectByPage")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "Web端管理员分页查询项目")
    public BaseResponse<Page<ProjectPageVO>> listProjectByPage(@Valid @RequestBody ProjectQueryRequest projectQueryRequest) {
        long current = projectQueryRequest.getCurrent();
        long pageSize = projectQueryRequest.getPageSize();
        QueryWrapper<Project> queryWrapper = projectService.getQueryWrapper(projectQueryRequest);
        Page<Project> page = projectService.page(new Page<>(current, pageSize), queryWrapper);
        List<Project> projectList = page.getRecords();
        List<ProjectPageVO> projectPageVOS = commonService.convertList(projectList, ProjectPageVO.class);
        Page<ProjectPageVO> voPage = new Page<>(current, pageSize);
        voPage.setRecords(projectPageVOS);
        voPage.setPages(page.getPages());
        voPage.setTotal(page.getTotal());
        return ResultUtils.success(voPage);
    }

    /**
     * Web端管理员上(下)架项目
     * @param commonRequest 项目id
     * @return 是否更新成功
     */
    @PostMapping("shelves")
    @Operation(summary = "Web端管理员上(下)架项目", description = "参数：项目id，权限：管理员(admin, boss)，方法名：updateProjectShelvesStatus")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目管理", content = "Web端管理员上(下)架项目")
    public BaseResponse<Boolean> updateProjectShelvesStatus(@Valid @RequestBody CommonRequest commonRequest) {
        // 获取当前服务类商品的上(下)架状态
        Long id = commonRequest.getId();
        Project project = projectService.getById(id);
        Boolean isShelves = project.getIsShelves();
        LambdaUpdateWrapper<Project> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(Project::getId, id).set(Project::getIsShelves, !isShelves);
        projectService.update(updateWrapper);
        return ResultUtils.success(true);
    }

//    /**
//     * Web端管理员根据项目id修改项目状态
//     * @param projectStatusUpdateRequest 项目状态更新请求体
//     * @return 是否更新成功
//     */
//    @PostMapping("status")
//    @Operation(summary = "Web端管理员根据项目id修改项目状态", description = "参数：项目id，权限：管理员(admin, boss)，方法名：updateProjectStatus")
//    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
//    @SysLog(title = "项目管理", content = "Web端管理员根据项目id修改项目状态")
//    public BaseResponse<Boolean> updateProjectStatus(@Valid @RequestBody ProjectStatusUpdateRequest projectStatusUpdateRequest) {
//        Long id = projectStatusUpdateRequest.getId();
//        String projectStatus = projectStatusUpdateRequest.getProjectStatus();
//        LambdaUpdateWrapper<Project> updateWrapper = new LambdaUpdateWrapper<>();
//        updateWrapper.eq(Project::getId, id).set(Project::getProjectStatus, projectStatus);
//        projectService.update(updateWrapper);
//        return ResultUtils.success(true);
//    }


}