package com.greenorange.promotion.controller.project;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.greenorange.promotion.annotation.RequiresPermission;
import com.greenorange.promotion.annotation.SysLog;
import com.greenorange.promotion.common.BaseResponse;
import com.greenorange.promotion.common.ErrorCode;
import com.greenorange.promotion.common.ResultUtils;
import com.greenorange.promotion.constant.UserConstant;
import com.greenorange.promotion.exception.ThrowUtils;
import com.greenorange.promotion.model.dto.projectCommission.ProjectCommissionAddRequest;
import com.greenorange.promotion.model.dto.projectDetail.ProjectDetailAddRequest;
import com.greenorange.promotion.model.dto.projectDetail.ProjectDetailUpdateRequest;
import com.greenorange.promotion.model.dto.subUserProjectCommission.SubUserProjectCommissionAddRequest;
import com.greenorange.promotion.model.entity.*;
import com.greenorange.promotion.model.vo.projectDetail.ProjectDetailVO;
import com.greenorange.promotion.service.common.CommonService;
import com.greenorange.promotion.service.project.ProjectCommissionService;
import com.greenorange.promotion.service.project.ProjectDetailService;
import com.greenorange.promotion.service.project.SubUserProjectCommissionService;
import com.greenorange.promotion.service.userInfo.UserInfoService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import com.greenorange.promotion.model.dto.CommonRequest;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 项目明细 控制器
 */
@RestController
@RequestMapping("projectDetail")
@Slf4j
@Tag(name = "项目明细管理")
public class ProjectDetailController {

    @Resource
    private ProjectDetailService projectDetailService;

    @Resource
    private CommonService commonService;

    @Resource
    private ProjectCommissionService projectCommissionService;

    @Resource
    private SubUserProjectCommissionService subUserProjectCommissionService;

    @Resource
    private UserInfoService userInfoService;

    /**
     * web端管理员添加项目明细
     * @param projectDetailAddRequest 项目明细添加请求体
     * @return 是否添加成功
     */
    @PostMapping("add")
    @Operation(summary = "web端管理员添加项目明细", description = "参数：项目明细添加请求体，权限：管理员，方法名：addProjectDetail")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目明细管理", content = "web端管理员添加项目明细")
    public BaseResponse<Boolean> addProjectDetail(@Valid @RequestBody ProjectDetailAddRequest projectDetailAddRequest) {
        ProjectDetail projectDetail = commonService.copyProperties(projectDetailAddRequest, ProjectDetail.class);
        projectDetailService.save(projectDetail);

        // 获取所有的小程序用户
        LambdaQueryWrapper<UserInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(UserInfo::getUserRole, UserConstant.DEFAULT_ROLE);
        List<UserInfo> userInfoList = userInfoService.list(lambdaQueryWrapper);

        // 获取参数信息
        List<ProjectCommissionAddRequest> projectCommissionAddRequestList = new ArrayList<>();
        Long projectDetailId = projectDetail.getId();
        BigDecimal projectSettlementPrice = projectDetail.getProjectSettlementPrice();
        Long projectId = projectDetail.getProjectId();

        // 给所有用户添加一条项目明细
        for (UserInfo userInfo : userInfoList) {
            ProjectCommissionAddRequest projectCommissionAddRequest = ProjectCommissionAddRequest.builder()
                    .projectDetailId(projectDetailId)
                    .myUnitPrice(projectSettlementPrice)
                    .projectId(projectId)
                    .userId(userInfo.getId())
                    .build();
            projectCommissionAddRequestList.add(projectCommissionAddRequest);
        }
        List<ProjectCommission> projectCommissions = commonService.convertList(projectCommissionAddRequestList, ProjectCommission.class);
        projectCommissionService.saveBatch(projectCommissions);

        // 给所有用户添加一条下级项目明细抽佣表
        userInfoList = userInfoList.stream().filter(userInfo -> userInfo.getParentUserId() != null).collect(Collectors.toList());
        List<SubUserProjectCommissionAddRequest> subUserProjectCommissionAddRequestList = new ArrayList<>();
        for (UserInfo userInfo : userInfoList) {
            SubUserProjectCommissionAddRequest  subUserProjectCommissionAddRequest = SubUserProjectCommissionAddRequest.builder()
                    .projectDetailId(projectDetailId)
                    .myUnitPrice(projectSettlementPrice)
                    .projectId(projectId)
                    .subUserId(userInfo.getId())
                    .userId(userInfo.getParentUserId())
                    .build();
            subUserProjectCommissionAddRequestList.add(subUserProjectCommissionAddRequest);
        }
        List<SubUserProjectCommission> subUserProjectCommissions = commonService.convertList(subUserProjectCommissionAddRequestList, SubUserProjectCommission.class);
        subUserProjectCommissionService.saveBatch(subUserProjectCommissions);

        return ResultUtils.success(true);
    }

    /**
     * web端管理员根据id修改项目明细信息
     * @param projectDetailUpdateRequest 项目明细更新请求体
     * @return 是否更新成功
     */
    @PostMapping("update")
    @Operation(summary = "web端管理员更新项目明细", description = "参数：项目明细更新请求体，权限：管理员，方法名：updateProjectDetail")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目明细管理", content = "web端管理员根据id修改项目明细信息")
    public BaseResponse<Boolean> updateProjectDetail(@Valid @RequestBody ProjectDetailUpdateRequest projectDetailUpdateRequest) {
        ProjectDetail projectDetail = commonService.copyProperties(projectDetailUpdateRequest, ProjectDetail.class);
        // 1.更新项目明细的结算价格
        projectDetailService.updateById(projectDetail);
        // 2.更新抽佣比例（如果抽佣比例比原来小）
        LambdaQueryWrapper<SubUserProjectCommission> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SubUserProjectCommission::getProjectDetailId, projectDetail.getId());
        List<SubUserProjectCommission> subUserProjectCommissionList = subUserProjectCommissionService.list(lambdaQueryWrapper);
        for (SubUserProjectCommission subUserProjectCommission : subUserProjectCommissionList) {
            BigDecimal currentCommissionRate = subUserProjectCommission.getCurrentCommissionRate();
            BigDecimal maxCommissionRate = projectDetail.getMaxCommissionRate();
            if (currentCommissionRate.compareTo(maxCommissionRate) > 0) subUserProjectCommission.setCurrentCommissionRate(maxCommissionRate);
        }
        // 3.用Map集合存储上级用户对下级用户的抽佣比例
        Map<String, BigDecimal> commissionRateMap = new HashMap<>();
        for (SubUserProjectCommission subUserProjectCommission : subUserProjectCommissionList) {
            Long userId = subUserProjectCommission.getUserId();
            Long subUserId = subUserProjectCommission.getSubUserId();
            BigDecimal currentCommissionRate = subUserProjectCommission.getCurrentCommissionRate();
            String key = userId + "-" + subUserId;
            commissionRateMap.put(key, currentCommissionRate);
        }
        // 获取所有的小程序用户
        LambdaQueryWrapper<UserInfo> userInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
        userInfoLambdaQueryWrapper.eq(UserInfo::getUserRole, UserConstant.DEFAULT_ROLE);
        List<UserInfo> userInfoList = userInfoService.list(userInfoLambdaQueryWrapper);
        // 用来存储每个用户的上级列表
        Map<Long, List<Long>> userParentMap = new HashMap<>();
        for (UserInfo userInfo : userInfoList) {
            String superUserList = userInfo.getSuperUserList();
            if (superUserList != null) userParentMap.put(userInfo.getId(), Arrays.stream(superUserList.split(",")).map(Long::parseLong).collect(Collectors.toList()));
        }
        // 4.更新所有用户的结算价格
        for (SubUserProjectCommission subUserProjectCommission : subUserProjectCommissionList) {
            Long userId = subUserProjectCommission.getUserId();
            List<Long> parentIds = userParentMap.get(userId);
            parentIds.add(userId);
            BigDecimal totalRate = BigDecimal.ONE;
            for (int i = 0; i < parentIds.size() - 1; i ++ ) {
                String key = parentIds.get(i) + "-" + parentIds.get(i + 1);
                BigDecimal commissionRate = commissionRateMap.get(key);
                totalRate = totalRate.multiply(BigDecimal.ONE.subtract(commissionRate));
            }
            BigDecimal projectSettlementPrice = projectDetail.getProjectSettlementPrice();
            BigDecimal projectMinSettlementPrice = projectDetail.getProjectMinSettlementPrice();
            BigDecimal finallySettlementPrice = projectSettlementPrice.multiply(totalRate);
            if (finallySettlementPrice.compareTo(projectMinSettlementPrice) < 0) finallySettlementPrice = projectMinSettlementPrice;
            subUserProjectCommission.setMyUnitPrice(finallySettlementPrice);
        }
        subUserProjectCommissionService.updateBatchById(subUserProjectCommissionList);

        return ResultUtils.success(true);
    }

    /**
     * web端管理员根据id删除项目明细
     * @param commonRequest 项目明细删除请求体
     * @return 是否删除成功
     */
    @PostMapping("delete")
    @Operation(summary = "web端管理员根据id删除项目明细", description = "参数：项目明细删除请求体，权限：管理员，方法名：delProjectDetail")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目明细管理", content = "web端管理员根据id删除项目明细")
    public BaseResponse<Boolean> delProjectDetail(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        projectDetailService.removeById(id);
        // 删除对应的项目明细抽佣记录
        LambdaQueryWrapper<ProjectCommission> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ProjectCommission::getProjectDetailId, id);
        projectCommissionService.remove(lambdaQueryWrapper);

        // 删除对应的下级用户的项目明细抽佣记录
        LambdaQueryWrapper<SubUserProjectCommission> subUserLambdaQueryWrapper = new LambdaQueryWrapper<>();
        subUserLambdaQueryWrapper.eq(SubUserProjectCommission::getProjectDetailId, id);
        subUserProjectCommissionService.remove(subUserLambdaQueryWrapper);
        return ResultUtils.success(true);
    }


    /**
     * web端管理员根据id查询项目明细
     * @param commonRequest 项目明细查询请求体
     * @return 项目明细信息
     */
    @PostMapping("queryById")
    @Operation(summary = "web端管理员根据id查询项目明细", description = "参数：项目明细查询请求体，权限：管理员，方法名：queryProjectDetailById")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目明细管理", content = "web端管理员根据id查询项目明细")
    public BaseResponse<ProjectDetailVO> queryProjectDetailById(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        ProjectDetail projectDetail = projectDetailService.getById(id);
        ThrowUtils.throwIf(projectDetail == null, ErrorCode.OPERATION_ERROR, "当前项目明细不存在");
        ProjectDetailVO projectDetailVO = commonService.copyProperties(projectDetail, ProjectDetailVO.class);
        return ResultUtils.success(projectDetailVO);
    }


    /**
     * web端管理员根据项目id查询项目明细
     * @param commonRequest 项目id
     * @return 项目明细列表
     */
    @PostMapping("query/pid")
    @Operation(summary = "web端管理员根据项目id查询项目明细", description = "参数：项目id，权限：管理员，方法名：queryProjectDetailByPid")
    @RequiresPermission(mustRole = UserConstant.ADMIN_ROLE)
    @SysLog(title = "项目明细管理", content = "web端管理员根据项目id查询项目明细")
    public BaseResponse<List<ProjectDetailVO>> queryProjectDetailByPid(@Valid @RequestBody CommonRequest commonRequest) {
        Long id = commonRequest.getId();
        LambdaQueryWrapper<ProjectDetail> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ProjectDetail::getProjectId, id);
        List<ProjectDetail> projectDetailList = projectDetailService.list(lambdaQueryWrapper);
        List<ProjectDetailVO> projectDetailVOS = commonService.convertList(projectDetailList, ProjectDetailVO.class);
        return ResultUtils.success(projectDetailVOS);
    }


}